/*
 *   Copyright 2011 Institute of Communication and Computer Systems (ICCS) 
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 */

#include "linked_lists/linked_lists.h"
#include "block_header.h"

void set_next(void *ptr, void *next_block) {
    get_header(ptr)->pointers.next = (list_node_ptr) next_block;
}

void * get_next(void *ptr) {
    return (void *) get_header(ptr)->pointers.next;
}

#ifdef BLOCKS_IN_DLL

void set_previous(void *ptr, void *previous_block) {
    if( ptr != NULL) {
        get_header(ptr)->pointers.previous = (list_node_ptr) previous_block;
    }
}

void * get_previous(void *ptr) {
    return (void *) get_header(ptr)->pointers.previous;
}

#endif /* BLOCKS_IN_DLL */

void remove_block(void *block, void *starting_node) {

    void *current_node, *previous_node;

    /* Traverse a list starting from the starting node until block is found. */
    for(current_node = starting_node; current_node != NULL; 
            current_node = get_next(current_node)) {
        if(current_node == block) {
            if(current_node == starting_node) {
                starting_node = get_next(block);
#ifdef BLOCKS_IN_DLL
                set_previous(starting_node, NULL);
#endif /* BLOCKS_IN_DLL */
            } else {
                set_next(previous_node, get_next(block));
#ifdef BLOCKS_IN_DLL
                set_previous(get_next(block), previous_node);
#endif /* BLOCKS_IN_DLL */
            }
            break;
        }
        previous_node = current_node;
    }
}
